#! /usr/bin/env python
# -*- coding: utf-8 -*-
################################################################################
#
#    An Indigo plugin (indigodomo.com) which fetches weather information
#    and stores it in custom created weather devices, for further usage
#
#    Copyright (C) 2014 Rudi Zengers, Forza Consiglio, Netherlands
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
#    More info and contact details can be found on the support website
#    at http://www.fcim.nl/index.php/software/91-weathermapplugin
#
#    Version history
#    ---------------
#    0.8 Oct 25, 2014 First full functional general release available
#    0.9 Nov 11, 2014 Parameterized number round off for temperature readings
#    1.0 Dec 19, 2014 First public released version 
#
################################################################################

import indigo
import os
import sys
import time
import urllib2
import simplejson as json

class Plugin(indigo.PluginBase):

	############################################################################
	# defines all the plugin's global variables
	############################################################################
	
	Version = "1.0" # Version of this plug in
	
	MaxDayCount = 5 # max amount of days we will retrieve data, see OWMdevids
	OWMdevids = [0,1,2,3,4,5] # max amount of days we can store data, see MaxDayCount
	Weekdays = [0,1,2,3,4,5,6] # days of the week
	MinimumDownloadInterval = 60 # minimum time in minutes between API calls
	ApiUrl = "http://api.openweathermap.org/data/2.5/" # where to get the data
	
	############################################################################
	# defines all the plugin's procedures
	############################################################################

	def __init__(self, pluginId, pluginDisplayName, pluginVersion, pluginPrefs):
		########################################################################
		# Initalization of the indigo base plugin and on top of it our plugin
		########################################################################
		
		indigo.PluginBase.__init__(self, pluginId, pluginDisplayName, \
										pluginVersion, pluginPrefs)
		self.debug = self.pluginPrefs.get("DebugOn",False)
		self.debugLog(u"init: debug flag seen")

	def __del__(self):
		########################################################################
		# Delete of our plugin from memory
		########################################################################
		
		return

	def startup(self):
		########################################################################
		# After the init we can start our plugin. Define actions here
		########################################################################
		
		self.debugLog(u"startup called")
		
		DayCount = int(self.pluginPrefs.get("DayCount", "5"))
		if (DayCount > Plugin.MaxDayCount):
			self.debugLog(u"startup: daycount %s corrected to max:  %s" % (DayCount, Plugin.MaxDayCount))
			DayCount = Plugin.MaxDayCount
				
		# Configure device #0 first which is the current weather device
		self.debugLog(u"startup: define device 0")
		Plugin.OWMdevids[0] = self.defineDevice(0,"Current Weather","FCIMcurrent")

		# Configure other devices which capture the forecast
		i = 0
		while (i < DayCount):
			i = i + 1
			self.debugLog(u"startup: define device %s" % i)
			Plugin.OWMdevids[i] = self.defineDevice(i,"Weather Forecast","FCIMforecast")

	def shutdown(self):
		########################################################################
		# Plugin is requested to shutdown
		########################################################################
		
		self.debugLog(u"shutdown called")

	def validatePrefsConfigUi(self, valuesDict):
		########################################################################
		# Validation of configuration input given. Is it within expected values?
		########################################################################
		
		errorDict = indigo.Dict()
		#CycleTime = int(self.pluginPrefs.get("MinutesTillNextRequest", Plugin.MinimumDownloadInterval)) * 60
		if ((int(valuesDict[u'DayCount']) < 1) | (int(valuesDict[u'DayCount']) > Plugin.MaxDayCount)):
			errorDict["DayCount"] = "Retrieve between 1 to %s days" % Plugin.MaxDayCount
			errorDict["showAlertText"] = "You cannot retrieve more than %s days forecast from the server" % Plugin.MaxDayCount
			return (False, valuesDict, errorDict)

		if int(valuesDict["MinutesTillNextRequest"]) < Plugin.MinimumDownloadInterval:
			errorDict["MinutesTillNextRequest"] = "A minimum interval of %s minutes is expected" % Plugin.MinutesTillNextRequest
			errorDict["showAlertText"] = "You cannot set the API interval time below %s minuts" % Plugin.MinutesTillNextRequest
			return(False, valuesDict, errorDict)
			
		# If we arrive here, both questions were positive and the configuration is valid
		return (True,  valuesDict, errorDict)

	def runConcurrentThread(self):
		########################################################################
		# A new thread is automatically created and runConcurrentThread() is 
		# called in that thread after startup() has been called. This function
		# will loop forever and only return after self.stopThread becomes True. 
		# This will be arranged by stopping Indigo or disabling the plugin.
		# If this function returns prematurely then the plugin host process will 
		# log an error and attempt to call runConcurrentThread() again after 
		# several seconds.
		########################################################################
		
		CycleTime = int(self.pluginPrefs.get("MinutesTillNextRequest", Plugin.MinimumDownloadInterval)) * 60
		self.debugLog(u"Weatherdata will be retrieved every %s seconds" % CycleTime)
		self.lastCheck = int(time.time()) - CycleTime - 1 # just to be sure
		
		try:
			while True:
				if (int(time.time()) - self.lastCheck) > CycleTime:
				
					# now it is time to get the weather data
					self.debugLog(u"time to receive the weather again")
					self.weatherdata()
					self.lastCheck = int(time.time())
				self.sleep(10) # seconds
				
		except self.StopThread:
			pass

	def kelvinToCelsius(Temperature):
		########################################################################
		# Accept a temperature in Kelvin and change it to Celsius
		########################################################################
		
		return Temperature - 273.15

	def defineDevice(self,devid,devdescription,devtypeid):
		########################################################################
		# Try to add the weather device. If they are already there from a
		# previous run, reuse them to prevent errors with scenarios, pages etc
		########################################################################
		
		mydev = "OWM Day %s" % devid
		self.debugLog(u"definedevice called for " + mydev)

		if mydev in indigo.devices:
		
			self.debugLog(u"definedevice found existing weather device " + \
				mydev + " and will re-use it")
			indigo.server.log(u"will re-use existing weather device " + mydev)
			Day = indigo.devices[mydev]
			
		else:
		
			self.debugLog(u"definedevice could not find existing weather " + \
				"device and will create " + mydev)
			indigo.server.log(u"creating weather device: " + mydev)
			Day = indigo.device.create(indigo.kProtocol.Plugin,
				address=str(devid), name=mydev,
				description=devdescription, deviceTypeId=devtypeid)
				
		return Day.id

	def windDegreeToText(self,deg,Languagecode):
		
		########################################################################
		# Try to add the weather device. If they are already there from a
		# previous run, reuse them to prevent errors with scenarios, pages etc
		########################################################################
		
		# Check wich language we want to return here
		if Languagecode == "de":
			wind_dir_name = ['N','NO','O','SO','S','SW','W','NW','N']
		if Languagecode == "nl":
			wind_dir_name = ['N','NO','O','ZO','Z','ZW','W','NW','N']
		if Languagecode == "en":
			wind_dir_name = ['N','NE','E','SE','S','SW','W','NW','N']
		
		wind_sections = 360 / 8
		offset = wind_sections / 2 
		y = int( (deg + offset) / wind_sections ) + 1
		if y>8:
			y=1
		return(y)

	def geturl(self,theUrl):
		########################################################################
		# Retrieve data from URL and return the answer in a json string
		########################################################################
		
		try:
			# fetch the data from the URL
			self.debugLog(u"Opening url %s" % theUrl)
			WeatherData = urllib2.urlopen(theUrl)
		except:
			self.debugLog(u"HTTPError %s " + err.message % err.code)
			indigo.server.log("Received HTTP error " + err.code + \
				" while opening " + OwUrl, isError=True)
			return false # no use to continue

		try:
			# If we arrived here, the data is retrieved with succes.
			# Lets try to decode in json format
			self.debugLog(u"Decoding the json answer")
			WeatherForeCast = json.load(WeatherData) # and store the json string in an object
		except:
			# includes simplejson.decoder.JSONDecodeError
			self.debugLog(u"SimpleJSON returned %s " + err.message % err.code)
			indigo.server.log("Decoding JSON has failed, error " + err.code + \
					" found", isError=True)
			return false # no use to continue

		if int(WeatherForeCast['cod']) != 200 :
			# if everything is ok then at least we received some data. We want
			# to have a status=200 to ensure everything went as expected
			self.debugLog(u"JSON structure [cod] contains error %s " % err.code)
			indigo.server.log(("Error (%s) occurred while loading " + \
					"the weather data" % WeatherForeCast['cod']),isError=True)
			return false
		return WeatherForeCast

	def SetWeek(self,mon,tue,wed,thu,fri,sat,sun):
		########################################################################
		# Translates input to an array
		########################################################################
		self.Weekdays[0] = mon
		self.Weekdays[1] = tue
		self.Weekdays[2] = wed
		self.Weekdays[3] = thu
		self.Weekdays[4] = fri
		self.Weekdays[5] = sat
		self.Weekdays[6] = sun
	
	def weekDay(self,Atime,Languagecode):
		########################################################################
		# Accept a date time in gmtime format (seconds since epoch and convert
		# that into a day of the week in the proper language and return that
		# to the calling party. Set Default to English
		########################################################################
		
	    SetWeek("Monday","Tuesday","Wednesday","Thursday","Friday","Saturday","Sunday")
		if Languagecode == "nl":
			SetWeek("Maandag","Dinsdag","Woensdag","Donderdag","Vrijdag","Zaterdag","Zondag")
		if Languagecode == "de":
			SetWeek("Montag","Dienstag","Mitwoch","Donnerstag","Freitag","Samstag","Sontag")
		if Languagecode == "it":
			SetWeek("Lunedi", "Martedì", "Mercoledì", "Giovedi", "Venerdì", "Sabato", "Domenica")
		if Languagecode == "es":
			SetWeek("Lunes", "Martes", "Miércoles", "Jueves", "Viernes", "Sábado", "Sun")
		if Languagecode == "ru":
			SetWeek("Понедельник", "Вторник", "Среда", "Четверг", "Пятница", "Суббота", "Воскресенье")
		if Languagecode == "tr":
			SetWeek("Pazartesi", "Salı", "Çarşamba", "Perşembe", "Cuma", "Cumartesi", "Pazar")
		if Languagecode == "sv":
			SetWeek("Måndag", "tisdag", "onsdag", "torsdag", "Friday", "Saturday", "Sunday")
		if Languagecode == "fr":
			SetWeek("Lundi", "mardi", "mercredi", "jeudi", "vendredi", "samedi", "Sunday")
		if Languagecode == "bg":
			SetWeek("Понеделник", "вторник", "сряда", "четвъртък", "петък", "Saturday", "неделя")
		if Languagecode == "fi":
			SetWeek("Monday", "tiistai", "keskiviikko", "torstai", "perjantai", "Saturday", "Sunnuntai")
		if Languagecode == "pl":
			SetWeek("Poniedziałek", "wtorek", "środa", "czwartek", "piątek", "sobota", "niedziela")
		if Languagecode == "pt":
			SetWeek("Monday", "terça-feira", "Quarta-feira", "Quinta-feira", "Friday", "Saturday", "Sunday")
		if Languagecode == "ca":
			SetWeek("Dilluns", "Dimarts", "Dimecres", "Dijous", "Divendres", "Dissabte", "Sunday")
		if Languagecode == "zh_tw":
			SetWeek(“星期一”，“星期二”，“星期三”，“日”，“星期五”，“星期六”，“星期天”)			
		if Languagecode == "zh":
			SetWeek(“星期一”，““星期一”，“星期二”，“星期三”，“日”，“星期五”，“星期六”，“星期天”)	
		if Languagecode == "ro":
			SetWeek("Luni", "Marți", "miercuri", "Joi", "Friday", "Saturday", "Duminica")
		if Languagecode == "hr":
			SetWeek("Ponedjeljak", "Tuesday", "srijeda", "Četvrtak", "Petak", "Saturday", "Nedjelja")
		if Languagecode == "uk":
			SetWeek("Понеділок", "Вівторок", "Середовище", "Четвер", "П'ятниця", "Субота", "Неділя")
								
		self.debugLog(u"weekDay, language = %s, first day of week: %s" % (Languagecode,self.Weekdays[0]))
		
		self.debugLog(u"weekday received time %s" % int(Atime))
		TodayDateTime = time.gmtime(Atime)
		self.debugLog(u"TodayDateTime = %s and weekday number %s" % (TodayDateTime,time.strftime("%w",TodayDateTime)))
		
		return self.Weekdays[int(time.strftime("%w",TodayDateTime))]

	def weatherdata(self):
		########################################################################
		# Check if we can fetch the weather data from the service
		########################################################################
		
		ApiKey = self.pluginPrefs.get("ApiKey", "0")
		if len(ApiKey) < 2:
			self.debugLog(u"len(Apikey)< 2, value read [" + ApiKey + "]")
			indigo.server.log("Cannot fetch data (yet) because Api key is not " + \
					"configured in plugin",isError=True)
			return
		
		CityID = self.pluginPrefs.get("CityId", "0")
		if len(CityID) < 2:
			self.debugLog(u"len(CityID)<2, value read [" + CityID + "]")
			indigo.server.log("Cannot fetch data (yet) because City ID is not " + \
				"configured in plugin",isError=True)
			return
		
		# CityID and ApiKey seems OK, lets get the weather
		self.debugLog(u"CityID>2 and ApiKey>2 so lets retrieve the weatherdata")
		self.retrieveWeatherdata(ApiKey,CityID)

	def retrieveWeatherdata(self,ApiKey,CityID):
		########################################################################
		# Fetch the weather data from the service and store it in the devices
		########################################################################
		
		self.debugLog(u"retrieveweatherdata retrieve prefs")
		
		# Our preferences
		Language = self.pluginPrefs.get("Language", "en")
		RoundNumbers = int(self.pluginPrefs.get("RoundNumbers", "1"))
		#Weekday = self.translateweekdays(Language)
		UnitSystem = self.pluginPrefs.get("MetricSystem", "Metric")
		DayCount = int(self.pluginPrefs.get("DayCount", "5"))
		
		# Build the urls where we want to retrieve the Json output
		FcUrl = self.ApiUrl + "forecast/daily?id=" + \
				CityID + "&APPID=" + ApiKey + \
				"&lang=" + Language + "&units=" + UnitSystem + \
				"&cnt=" + str(DayCount)
		
		CrUrl = self.ApiUrl + "weather?id=" + \
				CityID + "&APPID=" + ApiKey + \
				"&lang=" + Language + "&units=" + UnitSystem
		
		#Tell where we are
		self.debugLog(u"weatherdata parameters: ")
		self.debugLog(u"Api Url  = %s" % self.ApiUrl)
		self.debugLog(u"Forecast = %s" % FcUrl)
		self.debugLog(u"Current  = %s" % CrUrl)
		self.debugLog(u"Language = %s" % Language)
		self.debugLog(u"Api Key  = %s" % ApiKey)
		self.debugLog(u"Units    = %s" % UnitSystem)
		self.debugLog(u"City ID  = %s" % CityID)
		self.debugLog(u"Day Count= %s" % DayCount)
		
		# Retrieve the current weather
		
		WeatherCurrent = self.geturl(CrUrl)
		if WeatherCurrent == False:
			return # no use to continue
		
		# Store the current info first
		# ----------------------------
		# {"coord":{"lon":139,"lat":35},
		# "sys":{"country":"JP","sunrise":1369769524,"sunset":1369821049},
		# "weather":[{"id":804,"main":"clouds","description":"overcast clouds","icon":"04n"}],
		# "main":{"temp":289.5,"humidity":89,"pressure":1013,"temp_min":287.04,"temp_max":292.04},
		# "wind":{"speed":7.31,"deg":187.002},
		# "rain":{"3h":0},
		# "clouds":{"all":92},
		# "dt":1369824698,
		# "id":1851632,
		# "name":"Shuzenji",
		# "cod":200}
		
		dev=indigo.devices[Plugin.OWMdevids[0]]
		self.debugLog(u"Storing information for device " + dev.name)
	
		dev.updateStateOnServer("dateTime", WeatherCurrent['dt']) # date and time in integer format
		dev.updateStateOnServer("temperature", round(WeatherCurrent['main']['temp'],1))
		dev.updateStateOnServer("minTemperature", round(WeatherCurrent['main']['temp_min'],1))
		dev.updateStateOnServer("maxTemperature", round(WeatherCurrent['main']['temp_max'],1))
		
		dev.updateStateOnServer("pressure", WeatherCurrent['main']['pressure'])
		dev.updateStateOnServer("humidity", WeatherCurrent['main']['humidity'])		
		dev.updateStateOnServer("windSpeed", round(WeatherCurrent['wind']['speed'],1))
		dev.updateStateOnServer("windDegree", self.windDegreeToText(WeatherCurrent['wind']['deg'],Language))
		dev.updateStateOnServer("clouds", WeatherCurrent['clouds']['all'])	
				
		dev.updateStateOnServer("descriptionShort",WeatherCurrent['weather'][0]['main'])
		dev.updateStateOnServer("descriptionLong",WeatherCurrent['weather'][0]['description'])		
		dev.updateStateOnServer("descriptionId",WeatherCurrent['weather'][0]['id'])
		dev.updateStateOnServer("weatherIcon",WeatherCurrent['weather'][0]['icon'])
		self.debugLog(u"Weather stored, today temp: %s" % WeatherCurrent['main']['temp'])
		
		# wait moment and retrieve the next url
		
		self.debugLog(u"sleep 30 seconds")
		self.sleep(30) 
		
		# Retrieve the forecast info
		
		WeatherForeCast = self.geturl(FcUrl)
		if WeatherForeCast == False:
			return # no use to continue
		
		# {"cod":"200","message":0.0045,
		# "city":{"id":1851632,"name":"Shuzenji",
		# "coord":{"lon":138.933334,"lat":34.966671},
		# "country":"JP"},
		# "cnt":38,
		# "list":[{
		#      "dt":1406106000,
		#        "main":{
		#            "temp":298.77,
		#            "temp_min":298.77,
		#            "temp_max":298.774,
		#            "pressure":1005.93,
		#            "sea_level":1018.18,
		#            "grnd_level":1005.93,
		#            "humidity":87},
		#        "weather":[{"id":804,"main":"Clouds","description":"overcast clouds","icon":"04d"}],
		#        "clouds":{"all":88},
		#        "wind":{"speed":5.71,"deg":229.501},
		#        "sys":{"pod":"d"},
		#        "dt_txt":"2014-07-23 09:00:00"}
		#        ]}
		
		i = 0
		while (i < DayCount):
			i = i + 1
			dev=indigo.devices[Plugin.OWMdevids[i]]
			self.debugLog(u"Storing information for device " + dev.name)
			
			dev.updateStateOnServer("dateTime", self.weekDay(WeatherForeCast['list'][i-1]['dt'],Language))
			
			dev.updateStateOnServer("dayTemperature", round(WeatherForeCast['list'][i-1]['temp']['day'],RoundNumbers))
			dev.updateStateOnServer("minTemperature", round(WeatherForeCast['list'][i-1]['temp']['min'],RoundNumbers))
			dev.updateStateOnServer("maxTemperature", round(WeatherForeCast['list'][i-1]['temp']['max'],RoundNumbers))
			dev.updateStateOnServer("nightTemperature", round(WeatherForeCast['list'][i-1]['temp']['night'],RoundNumbers))
			dev.updateStateOnServer("eveningTemperature", round(WeatherForeCast['list'][i-1]['temp']['eve'],RoundNumbers))
			dev.updateStateOnServer("morningTemperature", round(WeatherForeCast['list'][i-1]['temp']['morn'],RoundNumbers))
			
			dev.updateStateOnServer("pressure", WeatherForeCast['list'][i-1]['pressure'])
			dev.updateStateOnServer("humidity", WeatherForeCast['list'][i-1]['humidity'])
			dev.updateStateOnServer("windSpeed", round(WeatherForeCast['list'][i-1]['speed'],RoundNumbers))
			dev.updateStateOnServer("windDegree", WeatherForeCast['list'][i-1]['deg'])
			#dev.updateStateOnServer("clouds", WeatherForeCast['list'][i-1]['clouds']['all'])
			
			dev.updateStateOnServer("descriptionShort",WeatherForeCast['list'][i-1]['weather'][0]['main'])
			dev.updateStateOnServer("descriptionLong",WeatherForeCast['list'][i-1]['weather'][0]['description'])
			dev.updateStateOnServer("descriptionId",WeatherForeCast['list'][i-1]['weather'][0]['id'])
			dev.updateStateOnServer("weatherIcon",WeatherForeCast['list'][i-1]['weather'][0]['icon'])
			
			self.debugLog(u"Weather stored, temp day %s: %s" % (i,WeatherForeCast['list'][i-1]['temp']['day']))
		indigo.server.log(u"Weather stored, today temp: %s" % round(WeatherCurrent['main']['temp'],RoundNumbers))

################################################################################
#   We are done
################################################################################
